
-- File: ~/.xmonad/xmonad.hs
-- Date: Sept 14, 2010
-- Description: User configuration file for xmonad

-- GENERIC --

--
-- xmonad example config file for xmonad-0.9
--
-- A template showing all available configuration hooks,
-- and how to override the defaults in your own xmonad.hs conf file.
--
-- Normally, you'd only override those defaults you care about.
--
-- NOTE: Those updating from earlier xmonad versions, who use
-- EwmhDesktops, safeSpawn, WindowGo, or the simple-status-bar
-- setup functions (dzen, xmobar) probably need to change
-- xmonad.hs, please see the notes below, or the following
-- link for more details:
--
-- http://www.haskell.org/haskellwiki/Xmonad/Notable_changes_since_0.8
--


import System.IO
import System.Exit
import Data.Monoid
import qualified Data.Map as M

import XMonad
import XMonad.Actions.UpdatePointer
import XMonad.Actions.GridSelect
import XMonad.Actions.FloatKeys

import XMonad.Util.Run(spawnPipe)
import XMonad.Util.Dmenu
import XMonad.Util.NamedScratchpad

import XMonad.Layout.NoBorders(noBorders,smartBorders)
import XMonad.Layout.ResizableTile
import XMonad.Layout.Grid
import XMonad.Layout.ToggleLayouts
import XMonad.Layout.SubLayouts
import XMonad.Layout.WindowNavigation
import XMonad.Layout.BoringWindows
import XMonad.Layout.Tabbed
import XMonad.Layout.Circle
import XMonad.Layout.Spacing
  
import XMonad.Hooks.ManageDocks
import XMonad.Hooks.DynamicLog
import XMonad.Hooks.UrgencyHook
import XMonad.Hooks.ManageHelpers
import XMonad.Hooks.FadeInactive (setOpacity)

import qualified XMonad.StackSet as W

 
-- The preferred terminal program, which is used in a binding below and by
-- certain contrib modules.
--
myTerminal      = "urxvtc"
 
-- Whether focus follows the mouse pointer.
myFocusFollowsMouse :: Bool
myFocusFollowsMouse = False
 
-- Width of the window border in pixels.
--
myBorderWidth   = 2
 
-- modMask lets you specify which modkey you want to use. The default
-- is mod1Mask ("left alt").  You may also consider using mod3Mask
-- ("right alt"), which does not conflict with emacs keybindings. The
-- "windows key" is usually mod4Mask.
--
myModMask       = mod4Mask
 
-- NOTE: from 0.9.1 on numlock mask is set automatically. The numlockMask
-- setting should be removed from configs.
--
-- You can safely remove this even on earlier xmonad versions unless you
-- need to set it to something other than the default mod2Mask, (e.g. OSX).
--
-- The mask for the numlock key. Numlock status is "masked" from the
-- current modifier status, so the keybindings will work with numlock on or
-- off. You may need to change this on some systems.
--
-- You can find the numlock modifier by running "xmodmap" and looking for a
-- modifier with Num_Lock bound to it:
--
-- > $ xmodmap | grep Num
-- > mod2        Num_Lock (0x4d)
--
-- Set numlockMask = 0 if you don't have a numlock key, or want to treat
-- numlock status separately.
--
-- myNumlockMask   = mod2Mask -- deprecated in xmonad-0.9.1
------------------------------------------------------------
 
 
-- The default number of workspaces (virtual screens) and their names.
-- By default we use numeric strings, but any string may be used as a
-- workspace name. The number of workspaces is determined by the length
-- of this list.
--
-- A tagging example:
--
-- > workspaces = ["web", "irc", "code" ] ++ map show [4..9]
--
myWorkspaces    = ["1:general","2:hack","3:code","4:prog","5:web","6:com","7:net","8:media","9:status","0:mail","esc:hide","-:11","=:12","\\:13", "`:14"]
 
-- Border colors for unfocused and focused windows, respectively.
--
myNormalBorderColor  = "#3B615D" -- "#dddddd"
myFocusedBorderColor = "#2FED55" -- "#ff0000"
 
------------------------------------------------------------------------
-- Key bindings. Add, modify or remove key bindings here.
--
myKeys conf@(XConfig {XMonad.modMask = modm}) = M.fromList $
 
    -- launch a terminal
    [ ((modm .|. shiftMask, xK_Return), spawn $ XMonad.terminal conf)

    -- launch dmenu
    , ((modm,               xK_p     ), spawn "dmenu_run -i -nb '#040404' -nf '#00FFFF' -sf '#ffa0ff' -sb '#000000'")
      
    -- close focused window
    , ((modm .|. shiftMask, xK_c     ), kill)
 
     -- Rotate through the available layout algorithms
    , ((modm,               xK_space ), sendMessage NextLayout)
 
    --  Reset the layouts on the current workspace to default
    , ((modm .|. shiftMask, xK_space ), setLayout $ XMonad.layoutHook conf)
 
    -- Resize viewed windows to the correct size
    , ((modm,               xK_n     ), refresh)
 
    -- Move focus to the next window
    , ((modm,               xK_Tab   ), windows W.focusDown)
 
    -- Move focus to the next window
    , ((modm,               xK_j     ), windows W.focusDown)
 
    -- Move focus to the previous window
    , ((modm,               xK_k     ), windows W.focusUp  )
 
    -- Move focus to the master window
    , ((modm,               xK_m     ), windows W.focusMaster  )
 
    -- Swap the focused window and the master window
    , ((modm,               xK_Return), windows W.swapMaster)
 
    -- Swap the focused window with the next window
    , ((modm .|. shiftMask, xK_j     ), windows W.swapDown  )
 
    -- Swap the focused window with the previous window
    , ((modm .|. shiftMask, xK_k     ), windows W.swapUp    )
 
    -- Shrink the master area
    , ((modm,               xK_h     ), sendMessage Shrink)
 
    -- Expand the master area
    , ((modm,               xK_l     ), sendMessage Expand)
 
    -- Push window back into tiling
    , ((modm,               xK_t     ), withFocused $ windows . W.sink)
 
    -- Increment the number of windows in the master area
    , ((modm              , xK_comma ), sendMessage (IncMasterN 1))
 
    -- Deincrement the number of windows in the master area
    , ((modm              , xK_period), sendMessage (IncMasterN (-1)))
 
    -- Toggle the status bar gap
    -- Use this binding with avoidStruts from Hooks.ManageDocks.
    -- See also the statusBar function from Hooks.DynamicLog.
    --
    , ((modm              , xK_b     ), sendMessage ToggleStruts)

    -- Quit xmonad (no questions)
    , ((modm .|. shiftMask .|. controlMask, xK_End), io (exitWith ExitSuccess))
 
    -- Quit xmonad (user friendly)
    , ((modm .|. shiftMask, xK_q), spawn "~/.bin/xmonadClose.sh")
      
    -- Unmount automounted disks by udiskie
    , ((modm .|. shiftMask .|. controlMask, xK_u), spawn "udiskie-umount -as && notify-send 'External media safely disconnected!'")
      
    --, ((modm .|. shiftMask, xK_w     ), if (dmenu "-b -i -nb '#040404' -nf '#525252' -sf '#ffa0ff' -sb '#000000' -p \"Do you really want to close XMonad? [yes/no]: \"") == "yes" then (sendMessage Expand) else (sendMessage Shrink))
      
    -- Restart xmonad
    , ((modm              , xK_q     ), spawn "pkill -9 xmobar; xmonad --recompile; xmonad --restart") ]
    ++
 
    --
    -- mod-[1..9], Switch to workspace N
    --
    -- mod-[1..9], Switch to workspace N
    -- mod-shift-[1..9], Move client to workspace N
    --
    [((m .|. modm, k), windows $ f i)
        | (i, k) <- zip (XMonad.workspaces conf) ([xK_1 .. xK_9] ++ [xK_0, xK_Escape, xK_minus, xK_equal, xK_backslash, xK_grave])
        , (f, m) <- [(W.greedyView, 0), (W.shift, shiftMask)]]
    
    -- Add custom key bindings
    ++ myGenericKeys

    ++
    -- Modification due to xinerama screens being in the wrong order
    [((m .|. modm, key), screenWorkspace sc >>= flip whenJust (windows . f))
        | (key, sc) <- zip [xK_w, xK_e, xK_r] [1,0,2] -- was [1,0,2] *** change to match your screen order ***
        , (f, m) <- [(W.view, 0), (W.shift, shiftMask)]]
 
-- Custom generic key bindings
-- Note: Use mod4Mask and controlMask to launch non-standard apps
myGenericKeys =

    -- Lock the screen using xscreensaver
    [ ((modm .|. shiftMask, xK_v), spawn "xscreensaver-command -lock")

    -- Toggle full-screen mode
    , ((modm .|. controlMask, xK_space), sendMessage (Toggle "Full"))
      
    -- Enable switching of windows with gridselect    
    , ((modm, xK_g), goToSelected defaultGSConfig)

    -- Enable switching of workspaces using gridselect
    , ((modm .|. shiftMask, xK_g), gridselectWorkspace defaultGSConfig W.view)
            
    -- Launch a backup terminal
    , ((modm .|. controlMask, xK_Return), spawn "xterm") 

    -- Launch firefox (now conkeror)
    , ((modm .|. controlMask, xK_f), spawn "conkeror")

    -- Launch emacs
    , ((modm .|. controlMask, xK_e), spawn "emacsclient -c")

    -- Launch tuxcmd
    --, ((modm .|. controlMask, xK_t), spawn "tuxcmd")
    
    -- Launch zathura
    , ((modm .|. controlMask, xK_z), spawn "zathura")

    -- Launch openoffice (now libreoffice)
    , ((modm .|. controlMask, xK_o), spawn "libreoffice")
      
    -- Launch vlc
    , ((modm .|. controlMask, xK_v), spawn "vlc")
      
    -- Launch a emacs scratchpad (key binding for HHKB)
    , ((modm .|. controlMask, xK_Escape), namedScratchpadAction scratchpads "emacs-scratch")

    -- Launch a emacs scratchpad (key binding for normal layout)
    , ((modm .|. controlMask, xK_grave), namedScratchpadAction scratchpads "emacs-scratch")

    -- Launch a screen scratchpad terminal
    , ((modm .|. controlMask, xK_1), namedScratchpadAction scratchpads "screen-terminal")
      
    -- Launch a maintainance scratchpad 
    , ((modm .|. controlMask, xK_2), namedScratchpadAction scratchpads "maintenance-terminal")

    -- Launch MC scratchpad
    , ((modm .|. controlMask, xK_3), namedScratchpadAction scratchpads "mc-scratch")

    -- Launch ncmpcpp scratchpad
    , ((modm .|. controlMask, xK_4), namedScratchpadAction scratchpads "ncmpcpp-scratch")
      
    -- Enable movement and resizing of floating windows
    , ((modm,               xK_d), withFocused (keysMoveWindow (10,0))) -- move floating window right
    , ((modm,               xK_a), withFocused (keysMoveWindow (-10,0))) -- move floating window left
    , ((modm,               xK_s), withFocused (keysMoveWindow (0,-10))) -- move floating window up      
    , ((modm,               xK_x), withFocused (keysMoveWindow (0,10))) -- move floating window down
    -- , ((modm .|. shiftMask, xK_a), withFocused (keysResizeWindow (-10,0) (0,0))) -- resize floating window (left)
    -- , ((modm .|. shiftMask, xK_d), withFocused (keysResizeWindow (10,0) (0,0))) -- resize floating window (right)
    -- , ((modm .|. shiftMask, xK_s), withFocused (keysResizeWindow (0,-10) (0,0))) -- resize floating window (up)
    -- , ((modm .|. shiftMask, xK_x), withFocused (keysResizeWindow (0,-10) (0,1))) -- resize floating window (down)
      
    -- For implementation of sub-layouts (not used currently)
    -- See http://xmonad.org/xmonad-docs/xmonad-contrib/XMonad-Layout-SubLayouts.html
    -- , ((modm .|. controlMask, xK_h), sendMessage $ pullGroup L)
    -- , ((modm .|. controlMask, xK_l), sendMessage $ pullGroup R)
    -- , ((modm .|. controlMask, xK_k), sendMessage $ pullGroup U)
    -- , ((modm .|. controlMask, xK_j), sendMessage $ pullGroup D)
      
    -- , ((modm .|. controlMask, xK_m), withFocused (sendMessage . MergeAll))
    -- , ((modm .|. controlMask, xK_u), withFocused (sendMessage . UnMerge))
      
    -- , ((modm .|. controlMask, xK_period), onGroup W.focusUp')
    -- , ((modm .|. controlMask, xK_comma), onGroup W.focusDown')
      
    -- Expand a mirror pane
    , ((modm .|. shiftMask, xK_l), sendMessage MirrorShrink) 

    -- Shrink a mirror pane
    , ((modm .|. shiftMask, xK_h), sendMessage MirrorExpand)]
  where modm = mod4Mask

------------------------------------------------------------------------
-- Mouse bindings: default actions bound to mouse events
--
myMouseBindings (XConfig {XMonad.modMask = modm}) = M.fromList $
 
    -- mod-button1, Set the window to floating mode and move by dragging
    [ ((modm, button1), (\w -> focus w >> mouseMoveWindow w
                                       >> windows W.shiftMaster))
 
    -- mod-button2, Raise the window to the top of the stack
    , ((modm, button2), (\w -> focus w >> windows W.shiftMaster))
 
    -- mod-button3, Set the window to floating mode and resize by dragging
    , ((modm, button3), (\w -> focus w >> mouseResizeWindow w
                                       >> windows W.shiftMaster))
 
    -- you may also bind events to the mouse scroll wheel (button4 and button5)
    ]
 
------------------------------------------------------------------------
-- Layouts:
 
-- You can specify and transform your layouts by modifying these values.
-- If you change layout bindings be sure to use 'mod-shift-space' after
-- restarting (with 'mod-q') to reset your layout state to the new
-- defaults, as xmonad preserves your old layout settings by default.
--
-- * NOTE: XMonad.Hooks.EwmhDesktops users must remove the obsolete
-- ewmhDesktopsLayout modifier from layoutHook. It no longer exists.
-- Instead use the 'ewmh' function from that module to modify your
-- defaultConfig as a whole. (See also logHook, handleEventHook, and
-- startupHook ewmh notes.)
--
-- The available layouts.  Note that each layout is separated by |||,
-- which denotes layout choice.
--
myLayout = smartSpacing 5 $ smartBorders . avoidStruts $
           toggleLayouts (noBorders Full) $
-- for sublayouts but not currently used..see myGenericBindings above
--           windowNavigation $ subTabbed $ boringWindows $
-- to get tabbed layout add simpleTabbed
           rztiled ||| Mirror rztiled ||| Grid ||| Circle
  where
    -- default tiling algorithm partitions the screen into two panes
    tiled   = Tall nmaster delta ratio
 
    -- ResizeableTall arguments
    rztiled = ResizableTall nmaster delta ratio []
    
    -- The default number of windows in the master pane
    nmaster = 1
 
    -- Default proportion of screen occupied by master pane
    ratio   = 1/2
 
    -- Percent of screen to increment by when resizing panes
    delta   = 3/100

------------------------------------------------------------------------
-- Thanks to OODavo from #haskell on freenode; used for applications
-- that do not doFullFloat well (they request a window size smaller
-- then the widgets they contain)
maxFloat = flip W.float $ rectWithBorder 0.05
    where rectWithBorder x = let lt = x
                                 wh = 1 - 2*x
                                 in W.RationalRect lt lt wh wh

doMaxFloat = ask >>= doF . maxFloat

-- removes duplicate code in my manage hook following the pattern:
-- resource =? "a" <||> resource =? "b" <||> ...
resourceIsOneOf = foldl (<||>) (return False) . fmap (resource =?)

------------------------------------------------------------------------
-- Window rules:
 
-- Execute arbitrary actions and WindowSet manipulations when managing
-- a new window. You can use this to, for example, always float a
-- particular program, or have a client always appear on a particular
-- workspace.
--
-- To find the property name associated with a program, use
-- > xprop | grep WM_CLASS
-- and click on the client you're interested in.
--
-- To match on the WM_NAME, you can use 'title' in the same way that
-- 'className' and 'resource' are used below.
--
myManageHook = composeAll
    [ resource =? "supertux2" --> doCenterFloat
    , title =? "DOOM 3" --> doFullFloat
    , resource =? "Qt-subapplication" <&&> title /=? "Oracle VM VirtualBox Manager" --> doFloat
    , resource =? "vncviewer" --> doCenterFloat
    , resource =? "opennx" --> doMaxFloat
    , resource =? "Steam" --> doCenterFloat
    , title =? "Xnest" --> doCenterFloat
    , resource =? "pavucontrol" --> doMaxFloat
    , resource =? "qemu-system-x86_64" --> doFullFloat
    -- , resource =? "Halo.exe" --> doCenterFloat
    -- , resource =? "UnrealTournament.exe" --> doCenterFloat
    -- , resource =? "hl2_linux" --> doFullFloat
    , isFullscreen --> (doF W.focusDown <+> doFullFloat)
    , resourceIsOneOf ["emacs","gvim"] --> ask >>= (liftX . flip setOpacity (15/16)) >> idHook
    , resource  =? "desktop_window" --> doIgnore ] <+> namedScratchpadManageHook scratchpads <+> manageDocks

-- NamedScratchPad Hook
scratchpads = [ NS "emacs-scratch" spawnEmacsScratch findEmacsScratch manageEmacsScratch
              , NS "maintenance-terminal" spawnMaintenanceTerminal findMaintenanceTerminal manageMaintenanceTerminal
              , NS "screen-terminal" spawnScreenTerminal findScreenTerminal manageScreenTerminal
              , NS "mc-scratch" spawnMcScratch findMcScratch manageMcScratch
              , NS "ncmpcpp-scratch" spawnNcmpcppScratch findNcmpcppScratch manageNcmpcppScratch]
  where
    findEmacsScratch = resource =? "emacs-scratch"
    findMaintenanceTerminal = resource =? "scratchpad"
    findScreenTerminal = resource =? "screen-scratch"
    findMcScratch = resource =? "mc-scratch"
    findNcmpcppScratch = resource =? "ncmpcpp-scratch"

    spawnEmacsScratch = myTerminal ++ " -name emacs-scratch -pe -tabbedex -e tmux new-session -s emacs-scratch \"emacsclient -nw\""
    spawnMaintenanceTerminal = myTerminal ++ " -name scratchpad -bg black"
    spawnScreenTerminal = myTerminal ++ " -name screen-scratch -bg black"
    spawnMcScratch = myTerminal ++ " -name mc-scratch -pe -tabbedex -e mc"
    spawnNcmpcppScratch = myTerminal ++ " -name ncmpcpp-scratch -pe -tabbedex -e ncmpcpp"

    manageEmacsScratch = customFloating $ W.RationalRect 0.10 0.10 0.80 0.80
    manageMaintenanceTerminal = customFloating $ W.RationalRect 0 0.66 1 0.34
    manageScreenTerminal = customFloating $ W.RationalRect 0 0.015 1 0.985
    manageMcScratch = customFloating $ W.RationalRect 0.20 0.15 0.60 0.70
    manageNcmpcppScratch = customFloating $ W.RationalRect 0.225 0.10 0.55 0.80

    -- manageScreenTerminal = customFloating $ W.RationalRect l t w h
    --   where
    --     h = 1        -- terminal height (%)
    --     w = 1        -- terminal width (%)
    --     t = 1 - h    -- distance from top edge (%)
    --     l = 1 - w    -- distance from left edge (%)

------------------------------------------------------------------------
-- Event handling
 
-- Defines a custom handler function for X Events. The function should
-- return (All True) if the default handler is to be run afterwards. To
-- combine event hooks use mappend or mconcat from Data.Monoid.
--
-- * NOTE: EwmhDesktops users should use the 'ewmh' function from
-- XMonad.Hooks.EwmhDesktops to modify their defaultConfig as a whole.
-- It will add EWMH event handling to your custom event hooks by
-- combining them with ewmhDesktopsEventHook.
--
myEventHook = mempty
 
------------------------------------------------------------------------
-- Status bars and logging
 
-- Perform an arbitrary action on each internal state change or X event.
-- See the 'XMonad.Hooks.DynamicLog' extension for examples.
--
--
-- * NOTE: EwmhDesktops users should use the 'ewmh' function from
-- XMonad.Hooks.EwmhDesktops to modify their defaultConfig as a whole.
-- It will add EWMH logHook actions to your custom log hook by
-- combining it with ewmhDesktopsLogHook.
--
myLogHook h = dynamicLogWithPP $ xmobarPP
	    { ppOutput = hPutStrLn h
            , ppUrgent = xmobarColor "yellow" "red" . xmobarStrip
	    , ppHidden = (\ ws -> "") }
	    
--	   where
--	   noScratchPad ws = if ws == "NSP" then "" else ws
-- TODO: hide workspace esc:hide; if (ws == "NSP" || ws == "esc:hide") then "" else ws)

------------------------------------------------------------------------
-- Startup hook
 
-- Perform an arbitrary action each time xmonad starts or is restarted
-- with mod-q.  Used by, e.g., XMonad.Layout.PerWorkspace to initialize
-- per-workspace layout choices.
--
-- By default, do nothing.
--
-- * NOTE: EwmhDesktops users should use the 'ewmh' function from
-- XMonad.Hooks.EwmhDesktops to modify their defaultConfig as a whole.
-- It will add initialization of EWMH support to your custom startup
-- hook by combining it with ewmhDesktopsStartup.
--
myStartupHook = return ()
 
------------------------------------------------------------------------
-- Now run xmonad with all the defaults we set up.
 
-- Run xmonad with the settings you specify. No need to modify this.
--
main = do
     xmproc <- spawnPipe "xmobar"
     xmonad $ withUrgencyHook NoUrgencyHook $ defaults xmproc
 
-- A structure containing your configuration settings, overriding
-- fields in the default config. Any you don't override, will
-- use the defaults defined in xmonad/XMonad/Config.hs
--
-- No need to modify this.
--
defaults sxmproc = defaultConfig {
      -- simple stuff
        terminal           = myTerminal,
        focusFollowsMouse  = myFocusFollowsMouse,
        borderWidth        = myBorderWidth,
        modMask            = myModMask,
        -- numlockMask deprecated in 0.9.1
        -- numlockMask        = myNumlockMask,
        workspaces         = myWorkspaces,
        normalBorderColor  = myNormalBorderColor,
        focusedBorderColor = myFocusedBorderColor,
 
      -- key bindings
        keys               = myKeys,
        mouseBindings      = myMouseBindings,
 
      -- hooks, layouts
        layoutHook         = myLayout,
        manageHook         = myManageHook,
        handleEventHook    = myEventHook,
        logHook            = myLogHook sxmproc >> updatePointer (Relative 0.5 0.5),
        startupHook        = myStartupHook
    }
